/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - Sumatra
 * Date: 22.07.2010
 * Author(s): Gero
 * 
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.sim.util.network;


import java.io.EOFException;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;

import org.apache.log4j.Logger;


/**
 * Receives a datagram-packet on the
 * 
 * @author Gero
 * 
 */
public class UnicastUDPReceiver implements IReceiver {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	protected final Logger	log				= Logger.getLogger(getClass());
	
	// Connection
	private DatagramSocket	socket			= null;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see UnicastUDPReceiver
	 * @param localPort
	 */
	public UnicastUDPReceiver(int localPort) {
		
		// Create socket
		boolean socketError = true;
		while (socket == null && socketError) { // Tries to open an socket until it works! =)
			try {
				this.socket = new DatagramSocket(localPort);
			} catch (SocketException err) {
				log.warn("Error while opening a socket on port: '" + localPort + "'! (UDP; maybe port blocked???)", err);
				localPort++;
				log.debug("Trying to open a socket on port '" + localPort + "'...");
			}
		}
	}
	

	UnicastUDPReceiver(DatagramSocket socket) {
		this.socket = socket;
	}
	

	// --------------------------------------------------------------------------
	// --- method ---------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public DatagramPacket receive(DatagramPacket store) throws IOException {
		try {
			socket.receive(store); // New data is written into the packets buffer
		} catch (EOFException eof) {
			log.error("EOF error, buffer may be too small!", eof);
		}
		
		return store;
	}
	

	@Override
	public void cleanup() throws IOException {
		if (socket != null) {
			socket.close(); // May cause socket to throw IOException; think about it in calling class!
			socket = null;
		}
	}
	

	// --------------------------------------------------------------------------
	// --- getter/setter --------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public InetAddress getLocalAddress() {
		return socket.getLocalAddress();
	}
	

	@Override
	public int getLocalPort() {
		return socket.getLocalPort();
	}

	DatagramSocket getSocket() {
		return socket;
	}
}
